// Vertex shader

struct CameraUniform {
	view_proj: mat4x4<f32>,
};

@group(1) @binding(0)
var<uniform> camera: CameraUniform;

struct VertexInput {
	@location(0) position: vec2<f32>,
	@location(1) tex_coords: vec2<f32>,
};

struct VertexOutput {
	@builtin(position) position: vec4<f32>,
	@location(0) tex_coords: vec2<f32>,
};

@vertex
fn vs_main(
	model: VertexInput,
) -> VertexOutput {
	var out: VertexOutput;
	out.tex_coords = model.tex_coords;
	out.position = camera.view_proj * vec4<f32>(model.position, 0.0, 1.0);
	return out;
}

// Fragment shader

struct SpotlightUniform {
	cursor_position: vec2<f32>,
	size: f32,
	zoom_level: f32,
	strength: f32
}

@group(0) @binding(0)
var t_diffuse: texture_2d<f32>;
@group(0) @binding(1)
var s_diffuse: sampler;
@group(2) @binding(0)
var<uniform> spotlight: SpotlightUniform;

@fragment
fn fs_main(in: VertexOutput) -> @location(0) vec4<f32> {
	var cursor4 = vec4<f32>(spotlight.cursor_position, 0.0, 1.0);
	var texColor = textureSample(t_diffuse, s_diffuse, in.tex_coords);
	var shadowColor = vec4<f32>(0.0, 0.0, 0.0, 1.0);
	var dist_check = length(cursor4 - in.position) < (spotlight.size * spotlight.zoom_level);
	var spotlightColor = select(spotlight.strength, 0.0, dist_check);
	var finalColor = mix(texColor, shadowColor, spotlightColor);
	return finalColor;
}
